@@ -19,6 +19,7 @@
 #include <string.h>
 #include <time.h>
 
+#include "pi-source.h"
 #include "pi-socket.h"
 #include "pi-file.h"
 
@@ -167,7 +168,7 @@ pi_file_open (char *name)
 
   pf->for_writing = 0;
 
-  if ((pf->f = fopen (name, "r")) == NULL)
+  if ((pf->f = fopen (name, "rb")) == NULL)
     goto bad;
 
   fseek (pf->f, 0, SEEK_END);
@@ -347,6 +348,8 @@ pi_file_free (struct pi_file *pf)
     free (pf->entries);
   if (pf->file_name)
     free (pf->file_name);
+  if (pf->rbuf)
+    free(pf->rbuf);
   if (pf->tmpf)
     fclose (pf->tmpf);
   free (pf);
@@ -562,6 +565,13 @@ pi_file_set_app_info (struct pi_file *pf, void *data, int size)
 {
   void *p;
 
+  if (!size) {
+    if (pf->app_info)
+      free (pf->app_info);
+    pf->app_info_size = 0;
+    return (0);
+  }
+
   if ((p = malloc (size)) == NULL)
     return (-1);
   memcpy (p, data, size);
@@ -577,6 +587,13 @@ int
 pi_file_set_sort_info (struct pi_file *pf, void *data, int size)
 {
   void *p;
+ 
+  if (!size) {
+    if (pf->sort_info)
+      free (pf->sort_info);
+    pf->sort_info_size = 0;
+    return (0);
+  }
 
   if ((p = malloc (size)) == NULL)
     return (-1);
@@ -703,7 +720,7 @@ pi_file_close_for_write (struct pi_file *pf)
     return (-1);
   }
 
-  if ((f = fopen (pf->file_name, "w")) == NULL)
+  if ((f = fopen (pf->file_name, "wb")) == NULL)
     return (-1);
 
   ip = &pf->info;
@@ -780,7 +797,7 @@ int pi_file_retrieve(struct pi_file * pf, int socket, int cardno)
 {
   int db;
   int l,j;
-  char buffer[0xffff];
+  unsigned char buffer[0xffff];
   if(dlp_OpenDB(socket, cardno, dlpOpenRead, pf->info.name, &db)<0)
     return -1;
     
@@ -830,22 +847,34 @@ int pi_file_install(struct pi_file * pf, int socket, int cardno)
   int db;
   int l,j;
   int reset = 0;
+  int flags;
   void * buffer;
   
   /* Delete DB if it already exists */
   dlp_DeleteDB(socket, cardno, pf->info.name);
   
+  /* Set up DB flags */
+  flags = pf->info.flags;
+  if (strcmp(pf->info.name, "Graffiti ShortCuts ")==0) {
+    flags |= 0x8000; /* Rewrite an open DB */
+    reset = 1; /* To be on the safe side */
+  }
+  printf("Flags = %8.8X, name = '%s'\n", flags, pf->info.name);
+  
   /* Create DB*/
   if(dlp_CreateDB(socket, pf->info.creator, pf->info.type, cardno,
-                       pf->info.flags, pf->info.version,
+                       flags, pf->info.version,
                        pf->info.name, &db)<0)
-    return -1;
+   return -1;
     
   pi_file_get_app_info(pf, &buffer, &l);
   
   /* All system updates seen to have the 'ptch' type, so trigger a reboot on those */
   if (pf->info.creator == pi_mktag('p','t','c','h'))
     reset = 1; 
+   
+  if (pf->info.flags & dlpDBFlagReset)
+    reset = 1;
       
   if(l>0)
     dlp_WriteAppBlock(socket, db, buffer, l);
