@@ -6,9 +6,10 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include "pi-source.h"
 #include "pi-socket.h"
-#include "dlp.h"
-#include "address.h"
+#include "pi-dlp.h"
+#include "pi-address.h"
 
 /* Yet more hair: reorganize fields to match visible appearence */
 
@@ -18,6 +19,29 @@ int realentry[19] =   { 0, 1, 13, 2,
                         14, 15, 16, 17,
                         18 };
 
+int encodechars = 0;
+
+int inchar(FILE *in) {
+  int c;
+
+  c = getc(in);
+  if (encodechars && c == '\\') {
+    c = getc(in);
+    switch(c) {
+      case 'b':                 c = '\b';  break;
+      case 'f':                 c = '\f';  break;
+      case 'n':                 c = '\n';  break;
+      case 't':                 c = '\t';  break;
+      case 'r':                 c = '\r';  break;
+      case 'v':                 c = '\v';  break;
+      case '\\':                c = '\\';  break;
+      default:  ungetc(c, in);  c = '\\';  break;
+    }
+  }
+  return c;
+}
+	
+
 int read_field(char * dest, FILE * in) {
   int c;
   
@@ -26,16 +50,16 @@ int read_field(char * dest, FILE * in) {
   } while ((c != EOF) && ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r')));
   
   if (c == '"') {
-    c = getc(in);
+    c = inchar(in);
     
     while (c != EOF) {
       if (c == '"') {
-        c = getc(in);
+        c = inchar(in);
         if (c != '"')
           break;
       }
       *dest++ = c;
-      c = getc(in);
+      c = inchar(in);
     }
   } else {
     while (c != EOF) {
@@ -43,7 +67,7 @@ int read_field(char * dest, FILE * in) {
           break;
       }
       *dest++ = c;
-      c = getc(in);
+      c = inchar(in);
     }
   }
   *dest++ = '\0';
@@ -61,13 +85,33 @@ int read_field(char * dest, FILE * in) {
     return 0;
 }
 
+void outchar(char c, FILE *out) {
+
+  if (encodechars) {
+    switch(c) {
+      case '"':   putc('"',  out);   putc('"',  out);  break;
+      case '\b':  putc('\\', out);   putc('b',  out);  break;
+      case '\f':  putc('\\', out);   putc('f',  out);  break;
+      case '\n':  putc('\\', out);   putc('n',  out);  break;
+      case '\t':  putc('\\', out);   putc('t',  out);  break;
+      case '\r':  putc('\\', out);   putc('r',  out);  break;
+      case '\v':  putc('\\', out);   putc('v',  out);  break;
+      case '\\':  putc('\\', out);   putc('\\', out);  break;
+      default:    putc(c, out);                        break;
+    }
+  }
+  else {
+    putc(c, out);
+    if (c == '"')
+      putc('"', out);
+  }
+}
+
 int write_field(FILE * out, char * source, int more) {
 
   putc('"', out);
   while (*source) {
-    putc(*source, out);
-    if (*source == '"')
-      putc(*source, out);
+    outchar(*source, out);
     source++;
   }
   putc('"',out);
@@ -166,11 +210,11 @@ int read_file(FILE * in, int sd, int db, struct AddressAppInfo * aai)
     }
 #endif
 
-    pack_Address(&a, buf, &l);
+    pack_Address(&a, (unsigned char *)buf, &l);
 #ifdef DEBUG
     dumpdata(buf,l);
 #endif    
-    dlp_WriteRecord(sd, db, attribute, 0, category, buf, l, 0);
+    dlp_WriteRecord(sd, db, attribute, 0, category, (unsigned char *)buf, l, 0);
     
   } while(i >= 0);
   
@@ -187,12 +231,12 @@ int write_file(FILE * out, int sd, int db, struct AddressAppInfo * aai)
   int category,attribute;
   
   for(i=0;
-      (j=dlp_ReadRecordByIndex(sd, db, i, buf, 0, &l, &attribute, &category))>=0;
+      (j=dlp_ReadRecordByIndex(sd, db, i, (unsigned char *)buf, 0, &l, &attribute, &category))>=0;
       i++)
       {
     if (attribute & dlpRecAttrDeleted)
       continue;
-    unpack_Address(&a, buf, l);
+    unpack_Address(&a, (unsigned char *)buf, l);
 
 /* Simplified system */
 #if 0    
@@ -223,11 +267,20 @@ int write_file(FILE * out, int sd, int db, struct AddressAppInfo * aai)
     }
 
     for (j=0;j<19;j++) {
+#ifdef NOT_ALL_LABELS
       if (augment && (j >= 4) && (j<=8))
         if (a.phonelabel[j-4] != j-4)
           write_field(out, aai->phonelabels[a.phonelabel[j-4]], 2);
       if (a.entry[realentry[j]])
         write_field(out, a.entry[realentry[j]], 1);
+
+#else  /* print the phone labels if there is something in the field */
+      if (a.entry[realentry[j]]) {
+	if (augment && (j >= 4) && (j<=8))
+	  write_field(out, aai->phonelabels[a.phonelabel[j-4]], 2);
+        write_field(out, a.entry[realentry[j]], 1);
+      }
+#endif
       else
         write_field(out, "", 1);
     }
@@ -245,7 +298,7 @@ char * progname;
 
 void Help(void)
 {
-  fprintf(stderr,"usage:%s %s [-c category] [-a] -r|-w file\n",progname,TTYPrompt);
+  fprintf(stderr,"usage:%s %s [-c category] [-d category] [-a] [-e] -r|-w file\n",progname,TTYPrompt);
   exit(2);
 }
 
@@ -260,6 +313,7 @@ int main(int argc, char *argv[])
   char buf[0xffff];
   struct AddressAppInfo aai;
   char * defaultcategoryname = 0;
+  char * deletecategory = 0;
   int mode = 0;
   int c;
 #ifdef sun
@@ -273,10 +327,17 @@ int main(int argc, char *argv[])
     Help();
 
   optind = 2;
-  while (((c = getopt(argc, argv, "c:arw")) != -1) && (mode == 0)) {
+  while (((c = getopt(argc, argv, "ed:c:arw")) != -1) && (mode == 0)) {
     switch (c) {
     case 'a':
       augment = 1;
+      break;
+    case 'd':
+      deletecategory = optarg;
+      break;
+    case 'e':
+      encodechars = 1;
+      break;
     case 'c':
       defaultcategoryname = optarg;
       break;
@@ -294,12 +355,12 @@ int main(int argc, char *argv[])
   if (mode == 0)
     Help();
   
-  if (!(sd = pi_socket(AF_SLP, SOCK_STREAM, PF_PADP))) {
+  if (!(sd = pi_socket(PI_AF_SLP, PI_SOCK_STREAM, PI_PF_PADP))) {
     perror("pi_socket");
     exit(1);
   }
     
-  addr.sa_family = AF_SLP;
+  addr.sa_family = PI_AF_SLP;
   addr.port = 3;
   strcpy(addr.device,argv[1]);
   
@@ -334,8 +395,8 @@ int main(int argc, char *argv[])
     exit(1);
   }
   
-  l = dlp_ReadAppBlock(sd, db, 0, buf, 0xffff);
-  unpack_AddressAppInfo(&aai, buf, l);
+  l = dlp_ReadAppBlock(sd, db, 0, (unsigned char *)buf, 0xffff);
+  unpack_AddressAppInfo(&aai, (unsigned char *)buf, l);
 
   if (defaultcategoryname)
     defaultcategory = match_category(defaultcategoryname,&aai);
@@ -344,12 +405,26 @@ int main(int argc, char *argv[])
 
   if (mode == 2) { /* Write */
     FILE * f = fopen(argv[optind],"w");
+    if (f == NULL) {
+      sprintf(buf, "%s: %s", argv[0], argv[optind]);
+      perror(buf);
+      exit(1);
+    }
     write_file(f, sd, db, &aai);
+    if (deletecategory) 
+      dlp_DeleteCategory(sd, db, match_category(deletecategory,&aai));
     fclose(f);
   } else if (mode == 1) {
     FILE * f;
     while(optind < argc) {
       f = fopen(argv[optind],"r");
+      if (f == NULL) {
+        sprintf(buf, "%s: %s", argv[0], argv[optind]);
+        perror(buf);
+        continue;
+      }
+      if (deletecategory) 
+        dlp_DeleteCategory(sd, db, match_category(deletecategory,&aai));
       read_file(f, sd, db, &aai);
       fclose(f);
       optind++;
